<?php

namespace Drupal\commerce_rajaongkir_jne;

use Drupal\commerce_price\Price;
use Drupal\commerce_shipping\Entity\ShipmentInterface;
use Drupal\commerce_shipping\ShippingRate;
use Drupal\commerce_shipping\ShippingService;
use Drupal\physical\WeightUnit;

/**
 * Class JNERateRequest.
 *
 * @package Drupal\commerce_rajaongkir_jne
 */
class JNERateRequest extends JNERequest {
  /**
   * @var \Drupal\commerce_shipping\Entity\ShipmentInterface*/
  protected $commerce_shipment;

  /**
   * @var array*/
  protected $configuration;

  /**
   * Set the shipment for rate requests.
   *
   * @param \Drupal\commerce_shipping\Entity\ShipmentInterface $commerce_shipment
   *   A Drupal Commerce shipment entity.
   */
  public function setShipment(ShipmentInterface $commerce_shipment) {
    $this->commerce_shipment = $commerce_shipment;
  }

  /**
   * Returns an array of RajaOngkir country symbols keyed.
   */
  function getCountryCode($name) {
    $countries = array(
      'AD' => 189,
      'AE' => 178,
      'AF' => 2,
      'AG' => 7,
      'AI' => '',
      'AL' => 4,
      'AM' => 190,
      'AN' => 126,
      'AO' => 6,
      'AQ' => '',
      'AR' => 8,
      'AS' => 185,
      'AT' => 10,
      'AU' => 9,
      'AW' => 191,
      'AX' => '',
      'AZ' => 11,
      'BA' => 194,
      'BB' => 15,
      'BD' => 14,
      'BE' => 16,
      'BF' => 26,
      'BG' => 25,
      'BH' => 13,
      'BI' => 27,
      'BJ' => 18,
      'BL' => 221,
      'BM' => 19,
      'BN' => 24,
      'BO' => 21,
      'BQ' => '',
      'BR' => 23,
      'BS' => 12,
      'BT' => 20,
      'BV' => '',
      'BW' => 22,
      'BY' => 192,
      'BZ' => 17,
      'CA' => 30,
      'CC' => '',
      'CD' => 235,
      'CF' => 33,
      'CG' => 39,
      'CH' => 163,
      'CI' => 42,
      'CK' => 40,
      'CL' => 35,
      'CM' => 29,
      'CN' => 36,
      'CO' => 37,
      'CR' => 41,
      'CU' => 43,
      'CV' => 196,
      'CW' => 199,
      'CX' => '',
      'CY' => 44,
      'CZ' => 45,
      'DE' => 66,
      'DJ' => 48,
      'DK' => 47,
      'DM' => 49,
      'DO' => 50,
      'DZ' => 5,
      'EC' => 51,
      'EE' => 200,
      'EG' => 52,
      'EH' => '',
      'ER' => 55,
      'ES' => 156,
      'ET' => 56,
      'FI' => 59,
      'FJ' => 58,
      'FK' => 202,
      'FM' => '',
      'FO' => 57,
      'FR' => 61,
      'GA' => 64,
      'GB' => 69,
      'GD' => 72,
      'GE' => 203,
      'GF' => 62,
      'GG' => 204,
      'GH' => 67,
      'GI' => 68,
      'GL' => 71,
      'GM' => 65,
      'GN' => 76,
      'GP' => 73,
      'GQ' => 54,
      'GR' => 70,
      'GS' => '',
      'GT' => 75,
      'GU' => 74,
      'GW' => 77,
      'GY' => 78,
      'HK' => 81,
      'HM' => '',
      'HN' => 80,
      'HR' => 198,
      'HT' => 79,
      'HU' => 82,
      'ID' => '',
      'IE' => 87,
      'IL' => 88,
      'IM' => '',
      'IN' => 84,
      'IO' => '',
      'IQ' => 86,
      'IR' => 85,
      'IS' => 83,
      'IT' => 89,
      'JE' => 205,
      'JM' => 90,
      'JO' => 92,
      'JP' => 91,
      'KE' => 94,
      'KG' => '',
      'KH' => 28,
      'KI' => 95,
      'KM' => 38,
      'KN' => 223,
      'KP' => 46,
      'KR' => 96,
      'KW' => 97,
      'KY' => 31,
      'KZ' => 93,
      'LA' => 98,
      'LB' => 99,
      'LC' => 224,
      'LI' => 208,
      'LK' => 157,
      'LR' => 101,
      'LS' => 100,
      'LT' => 209,
      'LU' => 103,
      'LV' => 207,
      'LY' => 102,
      'MA' => 119,
      'MC' => 211,
      'MD' => 210,
      'ME' => 212,
      'MF' => 225,
      'MG' => 105,
      'MH' => '',
      'MK' => 60,
      'ML' => 110,
      'MM' => 121,
      'MN' => 117,
      'MO' => 104,
      'MP' => 112,
      'MQ' => 113,
      'MR' => 114,
      'MS' => 118,
      'MT' => 111,
      'MU' => 115,
      'MV' => 109,
      'MW' => 107,
      'MX' => 116,
      'MY' => 108,
      'MZ' => 120,
      'NA' => 122,
      'NC' => 127,
      'NE' => 130,
      'NF' => '',
      'NG' => 131,
      'NI' => 129,
      'NL' => 125,
      'NO' => 132,
      'NP' => 124,
      'NR' => 123,
      'NU' => 214,
      'NZ' => 128,
      'OM' => 133,
      'PA' => 135,
      'PE' => 138,
      'PF' => 63,
      'PG' => 136,
      'PH' => 139,
      'PK' => 134,
      'PL' => 140,
      'PM' => '',
      'PN' => '',
      'PR' => 215,
      'PS' => '',
      'PT' => 141,
      'PW' => '',
      'PY' => 137,
      'QA' => 142,
      'RE' => 216,
      'RO' => 143,
      'RS' => 218,
      'RU' => 144,
      'RW' => 145,
      'SA' => 147,
      'SB' => 154,
      'SC' => 150,
      'SD' => 159,
      'SE' => 162,
      'SG' => 152,
      'SH' => 158,
      'SI' => 219,
      'SJ' => '',
      'SK' => 153,
      'SL' => 151,
      'SM' => '',
      'SN' => 149,
      'SO' => 220,
      'SR' => 160,
      'SS' => 159,
      'ST' => '',
      'SV' => 53,
      'SX' => 225,
      'SY' => 164,
      'SZ' => 161,
      'TC' => 175,
      'TD' => 34,
      'TF' => '',
      'TG' => 169,
      'TH' => 167,
      'TJ' => 228,
      'TK' => '',
      'TL' => 168,
      'TM' => 229,
      'TN' => 173,
      'TO' => 170,
      'TR' => 174,
      'TT' => 172,
      'TV' => 176,
      'TW' => 165,
      'TZ' => 166,
      'UA' => 230,
      'UG' => 177,
      'UM' => '',
      'US' => 179,
      'UY' => 180,
      'UZ' => 231,
      'VA' => 182,
      'VC' => 226,
      'VE' => 183,
      'VG' => 232,
      'VI' => 233,
      'VN' => 184,
      'VU' => 181,
      'WF' => '',
      'WS' => 146,
      'YE' => 186,
      'YT' => '',
      'ZA' => 155,
      'ZM' => 187,
      'ZW' => 188,
    );

    return $countries[$name];
  }


  function getProvinceCode($name){
    $province = [
      'Bali' => 1,
      'Bangka Belitung' => 2,
      'Banten' => 3,
      'Bengkulu' => 4,
      'DI Yogyakarta' => 5,
      'DKI Jakarta' => 6,
      'Gorontalo' => 7,
      'Jambi' => 8,
      'Jawa Barat' => 9,
      'Jawa Tengah' => 10,
      'Jawa Timur' => 11,
      'Kalimantan Barat' => 12,
      'Kalimantan Selatan' => 13,
      'Kalimantan Tengah' => 14,
      'Kalimantan Timur' => 15,
      'Kalimantan Utara' => 16,
      'Kepulauan Riau' => 17,
      'Lampung' => 18,
      'Maluku' => 19,
      'Maluku Utara' => 20,
      'Nanggroe Aceh Darussalam (NAD)' => 21,
      'Nusa Tenggara Barat (NTB)' => 22,
      'Nusa Tenggara Timur (NTT)' => 23,
      'Papua' => 24,
      'Papua Barat' => 25,
      'Riau' => 26,
      'Sulawesi Barat' => 27,
      'Sulawesi Selatan' => 28,
      'Sulawesi Tengah' => 29,
      'Sulawesi Tenggara' => 30,
      'Sulawesi Utara' => 31,
      'Sumatera Barat' => 32,
      'Sumatera Selatan' => 33,
      'Sumatera Utara' => 34,
    ];

    return $province[$name];
  }

  function getCityCode($city, $province) {
    $config = \Drupal::config('commerce_rajaongkir.rajaongkirapikey');
    $api_key = $config->get('your_rajaongkir_api_key');

    $type = '';
    if (substr_count($city, '(Kota)') > 0) {
      $type = 'Kota';
      $city = substr($city, 0, -7);
    }
    if (substr_count($city, '(Kabupaten)') > 0) {
      $type = 'Kabupaten';
      $city = substr($city, 0, -12);
    }

    $client = \Drupal::httpClient();

    $request = $client->get("https://pro.rajaongkir.com/api/city?province=". $province, [
      'headers' => [
        'key' => $api_key,
      ],
    ]);

      $arrayResponse = json_decode($request->getBody(), TRUE);
      $cities = $arrayResponse['rajaongkir']['results'];

      if ($type != '') {
        $keys = array_keys(array_column($cities, 'city_name'), $city);
        foreach ($keys as $k) {
          if ($cities[$k]['city_name'] == $city and $cities[$k]['type'] == $type) {
            $key = $k;
          }
        }
      } else {
        $key = array_search($city, array_column($cities, 'city_name'));
      }

      return $cities[$key]['city_id'];
  }

  function getKecamatanCode($kecamatan, $city) {
    $config = \Drupal::config('commerce_rajaongkir.rajaongkirapikey');
    $api_key = $config->get('your_rajaongkir_api_key');

    $client = \Drupal::httpClient();

    $request = $client->get("https://pro.rajaongkir.com/api/subdistrict?city=". $city, [
      'headers' => [
        'key' => $api_key,
      ],
    ]);

    $arrayResponse = json_decode($request->getBody(), TRUE);
    $cities = $arrayResponse['rajaongkir']['results'];

    $key = array_search($kecamatan, array_column($cities, 'subdistrict_name'));

    return $cities[$key]['subdistrict_id'];
  }

  /**
   * Fetch rates from the JNE API.
   */
  public function getRates() {
    $config = \Drupal::config('commerce_rajaongkir.rajaongkirapikey');
    $api_key = $config->get('your_rajaongkir_api_key');

    // Validate a commerce shipment has been provided.
    if (empty($this->commerce_shipment)) {
      throw new \Exception('Shipment not provided');
    }

    $rates = [];

    $address = $this->commerce_shipment->getShippingProfile()->get('address')->first();
    $store_address = $this->commerce_shipment->getOrder()->getStore()->getAddress();

    $total_weight = 0;
    foreach ($this->commerce_shipment->getItems() as $delta => $shipment_item) {
      $weight = $shipment_item->getWeight()->convert(WeightUnit::GRAM);
      $total_weight += $weight->getNumber();
    }

    if (!empty($address)) {

      $country_code = $address->getCountryCode();
      $city = $address->getLocality();

      $store_province = $store_address->getAdministrativeArea();
      $store_province_id = $this->getProvinceCode($store_province);

      $store_city = $store_address->getLocality();
      $store_city_id = $this->getCityCode($store_city, $store_province_id);


      if ($country_code == 'ID') {
        if ($city != '') {
          $province = $address->getAdministrativeArea();
          $province_id = $this->getProvinceCode($province);
          $city_id = $this->getCityCode($city, $province_id);
          $kecamatan = $address->getDependentLocality();
          $kecamatan_id = $this->getKecamatanCode($kecamatan, $city_id);

          $client = \Drupal::httpClient();

          $request = $client->post("https://pro.rajaongkir.com/api/cost", [
            'form_params' => [
              'origin' => $store_city_id,
              'originType' => 'city',
              'destination' => $kecamatan_id,
              'destinationType' => 'subdistrict',
              'weight' => $total_weight,
              'courier' => 'jne',
            ],
            'headers' => [
              'key' => $api_key,
            ]
          ]);

          $arrayResponse = json_decode($request->getBody(), TRUE);

          $costs = [];

          foreach ($arrayResponse['rajaongkir']['results'][0]['costs'] as $rate) {
            if ($rate['cost'][0]['etd'] == '1-1') {
              $rate['cost'][0]['etd'] = 1;
            }
            $costs[] = [
              'service' => $rate['service'],
              'value' => $rate['cost'][0]['value'],
              'etd' => $rate['cost'][0]['etd'],
            ];
          }

          usort($costs, function($a, $b) {
            return $b['value'] <=> $a['value'];
          });

          foreach ($costs as $rate) {
            $service_code = $rate['service'];

            // Only add the rate if this service is enabled.
            if (!in_array($service_code, $this->configuration['services'])) {
              continue;
            }

            $price = new Price((string) $rate['value'], 'IDR');
            $shipping_service = new ShippingService(
              'JNE - ' . $rate['service'],
              'JNE - ' . $rate['service'] . ' (Estimasi ' . $rate['etd'] . ' hari)'
            );
            $rates[] = new ShippingRate(
              $service_code,
              $shipping_service,
              $price
            );
          }
        }
      }
      else {
        $client = \Drupal::httpClient();

        $request = $client->post("https://pro.rajaongkir.com/api/v2/internationalCost", [
          'form_params' => [
            'origin' => $store_city_id,
            'destination' => $this->getCountryCode($country_code),
            'weight' => $total_weight,
            'courier' => 'jne',
          ],
          'headers' => [
            'key' => $api_key,
          ]
        ]);

        $arrayResponse = json_decode($request->getBody(), TRUE);

        $costs = [];

        $currency_rate = $arrayResponse['rajaongkir']['currency']['value'];

        foreach ($arrayResponse['rajaongkir']['results'][0]['costs'] as $rate) {
          if ($rate['etd'] == '1-1') {
            $rate['etd'] = 1;
          }
          $costs[] = [
            'service' => $rate['service'],
            'value' => ceil($rate['cost'] * $currency_rate),
            'etd' => $rate['etd'],
          ];
        }

        usort($costs, function($a, $b) {
          return $b['value'] <=> $a['value'];
        });

        foreach ($costs as $rate) {
          $service_code = $rate['service'];

          // Only add the rate if this service is enabled.
          if (!in_array($service_code, $this->configuration['services'])) {
            continue;
          }

          $price = new Price((string) $rate['value'], 'IDR');
          $shipping_service = new ShippingService(
            'JNE - ' . $rate['service'],
            'JNE - ' . $rate['service'] . ' (Estimasi ' . $rate['etd'] . ' hari)'
          );
          $rates[] = new ShippingRate(
            $service_code,
            $shipping_service,
            $price
          );
        }
      }
    }

    return $rates;
  }

}
